' ----------------------------------------------------------------------------
' RunWithFileUrl.vbs: Start program with given file URL
' (c) 2012-2018 BSK Datentechnik GmbH
' Author: Hans Schmidts, BSK Datentechnik GmbH
' ----------------------------------------------------------------------------
'
' $Revision: 1.7 $
' $Date: 2018/03/13 10:45:16 $
' CVS $Source: /BSK/BSK_PRJ/scripts/vbs/RunWithFileUrl.vbs,v $
'
' History:
' V1.7  2018-03-13  BSK/HS
' - Output to log file added (see below if 0)
' - Comments adapted for open_with-7
' V1.6  2016-11-25  BSK/HS
' - Allow "*FreeCommander* /T"
' V1.5  2016-08-10  BSK/HS
' - convert (some) utf-8 to ansi
' V1.4  2015-07-31  BSK/HS
' - Allow FreeCommander.exe /T to open the directory in a new tab
' V1.3  2015-03-25  HaSchm
' - CVS checkin of 2013-09-04  HaSchm:
' - Insert "CW01\" if path starts with "\\root\loc\bbuv\"
' V1.2  2012-11-08  HaSchm
' - Hint on short path, FreeCommander and explorer view added
' V1.1  2012-11-07  HaSchm
' - Initial revision, based on "Explore*.vbs"
'
' Usage:
' - WScript RunWithFileUrl.vbs <program> <options> <file_url>
'   e.g.:
'   WScript RunWithFileUrl.vbs C:\totalcmd\TOTALCMD.EXE file://D:/dir
'   WScript RunWithFileUrl.vbs C:\Tools\FreeCommanderXE\FreeCommander.exe /T file://D:/dir
'   WScript RunWithFileUrl.vbs explorer.exe file://D:/dir
'
' Notes:
' - Especially useful with Firefox "OpenWith" Add-on:
'   - Install "https://addons.mozilla.org/en-US/firefox/addon/open-with"
'     (tested with "open_with-5.2.1-fx+sm.xpi", "open_with-6.8.5-fx+sm+tb.xpi"
'     and open_with-7.0.2-an+fx.xpi).
'   - Configuration see "Firefox Plugins\OpenWith\readme-bsk.txt"
'   - Examples (enter without the '- '):
'     - D:\TOOL\Scripts\RunWithFileUrl.vbs C:\totalcmd\TOTALCMD.EXE
'     - C:\UTIL\RunWithFileUrl.vbs C:\Tools\FreeCommanderXE\FreeCommander.exe
'     - C:\UTIL\RunWithFileUrl.vbs C:\Tools\FreeCommanderXE\FreeCommander.exe /T
'     - D:\TOOL\Scripts\RunWithFileUrl.vbs C:\WINDOWS\explorer.exe
'     - D:\TOOL\Scripts\RunWithFileUrl.vbs C:\WINDOWS\explorer.exe /e,
'     Notes:
'     - The standard OpenWith with with "Internet Explorer" can be used for
'       file and directory links: Files are downloaded, to selected folder or
'       to temporary folder to be opened with associated application,
'       directories are opened with "Windows Explorer".
'     - OpenWith with with "Windows Explorer" can be used for file and directory
'       links, where files are opened with associated application.
'     - OpenWith with with "FreeCommander" can be used for file and directory
'       links, where files are selected in the directory (not opened).
'       Note: With FreeCommander <= 771 the /T options does not work if
'       ::Computer is the a active tab.
'     - The behaviour with not existing or not via normal shares reachable
'       files or directories depends on the tool (e.g. "Windows Explorer"
'       simply opens e.g. "My Documents" without any error message :().
'     - Adapt the full path of RunWithFileUrl and programs as needed.
'     - No '"' allowed in this arguments list, so both the vbs and
'       program must be given without spaces (may be a limitation of
'       open_with-5.2.1-fx+sm.xpi). If your program is in a path with
'       spaces, simply use the short 16-bit DOS 8.3 version, e.g.:
'       c:\progra~1\totalcmd\totalcmd.exe
'       Use dir /x in a command shell to see the short names, or change to
'       the directory and type
'       for %P in (.) do @echo %~sP
'       to get the short name of the current path (.).
'     - Use FreeCommander.exe /T to open the directory in a new tab. This
'       script automatically adds the needed /C at end of command line.
'     - Option "/e," (really with the comma!) can be used to turn on explorer
'       view with tree, but showing the tree may take a while, especially
'       on network shares.
'   - You can change the position in the list as needed.
' ----------------------------------------------------------------------------
option explicit

Const ForReading = 1, ForAppending = 8

dim sProgAndOptions, sUrlPar
dim sUrl, sCmdLine, s, iPos, i
dim oShell, oFileSys
const sDefLogFileName = "%temp%\~RunWithFileUrl_log.txt"
dim sLogFileName, fiLogFile

Set oShell = WScript.CreateObject ("WScript.shell")
Set oFileSys = CreateObject("Scripting.FileSystemObject")

if WScript.Arguments.Count=0 then
  MsgBox "Arguments missing" & _
    vbCRLF & "Usage: WScript RunWithFileUrl.vbs <program> <options> <file_url>"
  WScript.quit(1)
end if
'MsgBox "WScript.Arguments(0) = " & WScript.Arguments(0) ' debug if startet in a cmd shell
if WScript.Arguments.Count=1 then
  sProgAndOptions = "C:\WINDOWS\explorer.exe /e," ' default value
  sUrlPar = WScript.Arguments(0)
else
  sProgAndOptions = """" & WScript.Arguments(0) & """"
  for i=1 to WScript.Arguments.Count-2
    sProgAndOptions = sProgAndOptions & " " & WScript.Arguments(i)
  next
  sUrlPar = WScript.Arguments(WScript.Arguments.Count-1)
end if

sUrl = sUrlPar
sUrl = Replace(sUrl, "/", "\")

s = LCase(Left(sUrl,5))  ' 5 = length("file:")
if s = "file:" then
  sUrl = Mid(sUrl,5+1)   ' remove "file:"
  do while Left(sUrl,1)="\"
    sUrl = Mid(sUrl,2)  ' remove other leading '\'
  loop
  ' decode percent-encoded URL
  sUrl = Replace(sUrl, "%20", " ")
  sUrl = Replace(sUrl, "%21", "!")
  sUrl = Replace(sUrl, "%23", "#")
  sUrl = Replace(sUrl, "%24", "$")
  sUrl = Replace(sUrl, "%26", "&")
  sUrl = Replace(sUrl, "%27", "'")
  sUrl = Replace(sUrl, "%28", "(")
  sUrl = Replace(sUrl, "%29", ")")
  sUrl = Replace(sUrl, "%2B", "+")
  sUrl = Replace(sUrl, "%2C", ",")
  sUrl = Replace(sUrl, "%3B", ";")
  sUrl = Replace(sUrl, "%3D", "=")
  sUrl = Replace(sUrl, "%40", "@")
  sUrl = Replace(sUrl, "%5B", "[")
  sUrl = Replace(sUrl, "%5D", "]")
  sUrl = Replace(sUrl, "%25", "%")

  ' convert (some) utf-8 to ansi
  sUrl = Replace(sUrl, "%C3%84", "")
  sUrl = Replace(sUrl, "%C3%96", "")
  sUrl = Replace(sUrl, "%C3%9C", "")
  sUrl = Replace(sUrl, "%C3%A4", "")
  sUrl = Replace(sUrl, "%C3%B6", "")
  sUrl = Replace(sUrl, "%C3%BC", "")
  sUrl = Replace(sUrl, "%C3%9F", "")
  sUrl = Replace(sUrl, "%C2%B5", "")
  sUrl = Replace(sUrl, "%C3%A9", "")
  sUrl = Replace(sUrl, "%C2%B2", "")
  sUrl = Replace(sUrl, "%C2%B3", "")
  sUrl = Replace(sUrl, "%C2%A7", "")
  sUrl = Replace(sUrl, "%E2%82%AC", "")

  if Mid(sUrl,2,1)<>":" then  ' not e.g "D:\DIR" (e.g. "host\share\dir")?
    ' URL does not start with drive letter and ':' -> add '\\'
    sUrl = "\\" & sUrl
  end if
end if

if Mid(sUrl,2,1)=":" then  ' is it e.g "D:\DIR"?
  ' URL start with drive letter and ':' -> check that no add. ':' is found
  ' and that it's an absolute path
  iPos = InStr(3,sUrl,":")
  if (iPos>0) or (Mid(sUrl,3,1)<>"\") then
    MsgBox "Invalid file url: " & sUrlPar
    WScript.quit(1)
  end if
else ' e.g. "host\share\dir"
  ' URL does not start with drive letter and ':' -> check that at least share
  ' given (check for path delimiter in URL)
  iPos = InStr(3,sUrl,"\")
  if (iPos<=0) or (Left(sUrl,2)<>"\\") then
    MsgBox "Invalid file url: " & sUrlPar
    WScript.quit(1)
  end if
  ' workaround for missing domain
  if LCase(Left(sUrl,16))="\\root\loc\bbuv\" then
    sUrl = "\\CW01\" & Mid(sUrl,3,9999)
  end if
end if

' start program with given file or dir
sCmdLine = sProgAndOptions & " """ & sUrl & """"
if (InStr(sProgAndOptions,"FreeCommander")>0) and (Right(sProgAndOptions,3)=" /T") then
  sCmdLine = sCmdLine & " /C"
end if
if 0 then ' debug log to temp file
  sLogFileName = oShell.ExpandEnvironmentStrings(sDefLogFileName)
  Set fiLogFile = oFileSys.OpenTextFile( sLogFileName, ForAppending, True )
  fiLogFile.WriteLine "[" & Date & " " & Time & "]"
  fiLogFile.WriteLine "sUrlPar = " & sUrlPar
  fiLogFile.WriteLine "sCmdLine = " & sCmdLine
  fiLogFile.Close
end if
'MsgBox "sCmdLine = " & sCmdLine ' debug if startet in a cmd shell
oShell.Run sCmdLine, 1, false ' false=don't wait
' intWindowsStyle: 0 = Hides the window and activates another window.
'                  1 = Activates and displays a window.
set oShell = nothing
set oFileSys = nothing
' eof
